"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.updateUserPackageJSON = exports.updatePackageJSONDependencies = exports.fakePackageJson = exports.checkStarterKytVersion = exports.addPackageJsonScripts = exports.addKytDependency = void 0;
var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));
var fs = require('fs');
var semver = require('semver');
var shell = require('shelljs');
var cloneDeep = require('lodash.clonedeep');
var logger = require('kyt-utils/logger');
var fakePackageJson = {
  name: '',
  version: '1.0.0',
  description: '',
  main: '',
  author: '',
  license: ''
};

// Add kyt to list of dev dependencies if its not there
exports.fakePackageJson = fakePackageJson;
var addKytDependency = function addKytDependency(packageJSON, kytPrefVersion) {
  // check to see if kyt is in dependencies or devDependencies
  if (!(packageJSON.dependencies && packageJSON.dependencies.kyt) && !(packageJSON.devDependencies && packageJSON.devDependencies.kyt)) {
    var kytVersion = kytPrefVersion;
    // If a version wasn't specified, install latest
    if (!kytVersion) {
      var output = shell.exec('npm info kyt version');
      kytVersion = output.stdout.trim();
    }
    if (packageJSON.dependencies && packageJSON.dependencies.kyt) {
      delete packageJSON.dependencies.kyt;
    }
    packageJSON.devDependencies = packageJSON.devDependencies || {};
    packageJSON.devDependencies.kyt = kytVersion;
  }
  return packageJSON;
};

// Adds kyt and Starter-kyt commands as npm scripts
exports.addKytDependency = addKytDependency;
var addPackageJsonScripts = function addPackageJsonScripts(userPackageJSON, starterPackageJSON) {
  var _starterPackageJSON$k;
  var starterKytConfig = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  if (!userPackageJSON.scripts) {
    userPackageJSON.scripts = {};
  }
  // for commands that aren't 1:1 name:script
  var commandMap = {
    dev: 'kyt dev',
    build: 'kyt build',
    'kyt:help': 'kyt --help'
  };
  if (starterKytConfig.hasServer !== false) {
    commandMap.start = 'node build/server/main.js';
  }
  var deprecatedConfig = (starterPackageJSON === null || starterPackageJSON === void 0 ? void 0 : (_starterPackageJSON$k = starterPackageJSON.kyt) === null || _starterPackageJSON$k === void 0 ? void 0 : _starterPackageJSON$k.scripts) || [];
  if (deprecatedConfig && deprecatedConfig.length > 0) {
    logger.error('Use of`kyt.scripts` is deprecated in `package.json`. You can simply declare `scripts`.');
    process.exit(1);
  }
  var starterScripts = starterPackageJSON && starterPackageJSON.scripts || {};

  // Merge the Starter-kyt script names into the list of commands.
  var keys = [].concat((0, _toConsumableArray2["default"])(Object.keys(commandMap)), (0, _toConsumableArray2["default"])(Object.keys(starterScripts)));
  var commands = new Set(keys);
  commands.forEach(function (command) {
    // If the command is from a starter-kyt then
    // we need to copy in the starter-kyt value.
    if (starterScripts[command]) {
      userPackageJSON.scripts[command] = starterScripts[command];
    } else if (!userPackageJSON.scripts[command]) {
      userPackageJSON.scripts[command] = commandMap[command];
    }
  });
  logger.task('Added kyt scripts into your package.json scripts');
  return userPackageJSON;
};

// Compare the starter-kyt's package.json kyt.version
// configuration to make sure kyt is an expected version.
exports.addPackageJsonScripts = addPackageJsonScripts;
var checkStarterKytVersion = function checkStarterKytVersion(userPackageJSON, starterPackageJSON) {
  var kytStarterPreferredVersion = starterPackageJSON.dependencies && starterPackageJSON.dependencies.kyt || starterPackageJSON.devDependencies && starterPackageJSON.devDependencies.kyt || null;
  if (kytStarterPreferredVersion) {
    // Look everywhere for kyt
    var kytVersion = userPackageJSON.devDependencies && userPackageJSON.devDependencies.kyt || userPackageJSON.dependencies && userPackageJSON.dependencies.kyt;
    if (semver.valid(kytVersion)) {
      if (!semver.satisfies(kytVersion, kytStarterPreferredVersion)) {
        logger.warn("".concat(starterPackageJSON.name, " requires kyt version ").concat(kytStarterPreferredVersion, " but kyt ").concat(kytVersion, " is installed."));
      }
    }
  }
  return kytStarterPreferredVersion;
};

// Adds dependencies from the starter-kyts package.json
exports.checkStarterKytVersion = checkStarterKytVersion;
var updatePackageJSONDependencies = function updatePackageJSONDependencies(packageJSON, starterPackageJSON) {
  ['resolutions', 'dependencies', 'devDependencies'].forEach(function (key) {
    if (!packageJSON[key] && !starterPackageJSON[key]) {
      return;
    }
    var deps = {};
    var keys = new Set([].concat((0, _toConsumableArray2["default"])(Object.keys(packageJSON[key] || {})), (0, _toConsumableArray2["default"])(Object.keys(starterPackageJSON[key] || {}))));
    keys.forEach(function (depKey) {
      deps[depKey] = starterPackageJSON[key] && starterPackageJSON[key][depKey] || packageJSON[key] && packageJSON[key][depKey];
    });
    packageJSON[key] = deps;
  });
  logger.task('Added new dependencies to package.json');
  return packageJSON;
};

// Add dependencies, scripts and other package to
// the user's package.json configuration.
exports.updatePackageJSONDependencies = updatePackageJSONDependencies;
var updateUserPackageJSON = function updateUserPackageJSON(starterPackageJSON, starterKytConfig, paths, kytVersion) {
  var newUserPackageJSON;
  // Create a package.json definition if
  // the user doesn't already have one.
  if (shell.test('-f', paths.userPackageJSONPath)) {
    var userJSON = fs.readFileSync(paths.userPackageJSONPath, 'utf8');
    newUserPackageJSON = JSON.parse(userJSON);
  } else {
    newUserPackageJSON = cloneDeep(fakePackageJson);
    logger.task('Creating a new package.json. You should fill it in.');
  }
  // Clone the package.json so that we have a backup.
  var oldUserPackageJSON = cloneDeep(newUserPackageJSON);

  // Add dependencies from starter-kyts
  var kytPrefVersion = kytVersion || checkStarterKytVersion(newUserPackageJSON, starterPackageJSON);
  newUserPackageJSON = updatePackageJSONDependencies(newUserPackageJSON, starterPackageJSON);
  newUserPackageJSON = addKytDependency(newUserPackageJSON, kytPrefVersion);
  newUserPackageJSON = addPackageJsonScripts(newUserPackageJSON, starterPackageJSON, starterKytConfig);
  fs.writeFileSync(paths.userPackageJSONPath, JSON.stringify(newUserPackageJSON, null, 2));
  return {
    oldUserPackageJSON: oldUserPackageJSON,
    newUserPackageJSON: newUserPackageJSON
  };
};
exports.updateUserPackageJSON = updateUserPackageJSON;